defmodule AdvertisementCreationContext do
  use WhiteBread.Context
  use Hound.Helpers
  alias PropTrackr.Accounts
  alias PropTrackr.Repo
  alias PropTrackr.Accounts.User
  alias Properties.Photo

  scenario_starting_state fn _state ->
    Ecto.Adapters.SQL.Sandbox.checkout(PropTrackr.Repo)
    Ecto.Adapters.SQL.Sandbox.mode(PropTrackr.Repo, {:shared, self()})
    Hound.start_session()
    %{}
  end

  scenario_finalize fn _status, _state ->
    Ecto.Adapters.SQL.Sandbox.checkin(PropTrackr.Repo)
    Hound.end_session()
  end

  given_ ~r/^there exists following accounts$/, fn state, %{table_data: table} ->
    table
    |> Enum.map(fn user_details -> User.changeset(%User{}, user_details) end)
    |> Enum.each(fn changeset -> Repo.insert!(changeset) end)

    existing_user = List.first(table)

    {
      :ok,
      state
      |> Map.put(:email, existing_user[:email])
      |> Map.put(:password, existing_user[:password])
    }
  end

  given_ ~r/^I want to insert the following data$/, fn state, %{table_data: table} ->
    data = List.first(table)
    {
      :ok,
      state
      |> Map.put(:data, data)
    }
  end

  and_ ~r/^I am logged in$/, fn state ->
    setup_session(state[:email], state[:password])
    {:ok, state}
  end

  and_ ~r/^I want to create a new property advertisement$/, fn state ->
    navigate_to("/")
    click({:id, "add_advertisement"})
    {:ok, state}
  end

  and_ ~r/^I fill in the property advertisement form with valid data$/, fn state ->
    fill_field({:id, "title"}, state[:data][:title])
    fill_field({:id, "description"}, state[:data][:description])
    select_dropdown("type", state[:data][:type])
    select_dropdown("property_type", state[:data][:property_type])
    fill_field({:id, "location"}, state[:data][:location])
    fill_field({:id, "area"}, state[:data][:area])
    fill_field({:id, "room_count"}, state[:data][:room_count])
    fill_field({:id, "floor"}, state[:data][:floor])
    fill_field({:id, "floor_count"}, state[:data][:floor_count])
    fill_field({:id, "price"}, state[:data][:price])
    {:ok, state}
  end

  and_ ~r/^I upload 3 valid photos$/, fn state ->
    # Find the file input element
    file_input = find_element(:id, "property_photos")

    execute_script("""
      (function() {
        const input = document.getElementById('property_photos');
        const files = [
          new File(['test content'], 'test_photo_1.jpg', { type: 'image/jpeg' }),
          new File(['test content'], 'test_photo_2.jpg', { type: 'image/jpeg' }),
          new File(['test content'], 'test_photo_3.jpg', { type: 'image/jpeg' })
        ];
        const dataTransfer = new DataTransfer();
        files.forEach(file => dataTransfer.items.add(file));
        input.files = dataTransfer.files;
        input.dispatchEvent(new Event('change', { bubbles: true }));
      })();
    """)

    # Give the browser a moment to process the file upload
    :timer.sleep(100)

    {:ok, state}
  end

  when_ ~r/^I click submit$/, fn state ->
    click({:id, "submit_button"})
    {:ok, state}
  end

  then_ ~r/^I should see a success message$/, fn state ->
    assert visible_in_page? ~r/Property created successfully!/
    {:ok, state}
  end

  and_ ~r/^I should be redirected to the property advertisement page$/, fn state ->
    paths = String.split(current_path(), "/")
    # https://stackoverflow.com/a/6640851
    regex = ~r/^[0-9a-f]{8}-[0-9a-f]{4}-[0-9a-f]{4}-[0-9a-f]{4}-[0-9a-f]{12}$/
    assert Enum.at(paths, -1) =~ regex
    {:ok, state}
  end

  and_ ~r/^I should see all uploaded photos on the details page$/, fn state ->
    # Wait for images to load
    :timer.sleep(500)

    # Find the photo slides container and count the photos
    photo_slides = find_all_elements(:css, ".photo-slide")

    # Verify navigation elements if there are multiple photos
    if length(photo_slides) > 1 do
      prev_button = find_element(:css, "button.prev-photo")
      next_button = find_element(:css, "button.next-photo")
      thumbnails = find_all_elements(:css, "button.thumbnail")

      # Check that we have the correct number of thumbnails
      assert length(thumbnails) == length(photo_slides),
        "Expected #{length(photo_slides)} thumbnails but found #{length(thumbnails)}"
    end

    # Verify each photo is loaded and visible
    Enum.each(photo_slides, fn slide ->
      img = find_within_element(slide, :css, "img")
      src = attribute_value(img, "src")

      assert String.contains?(src, "/uploads/"),
        "Image source should contain /uploads/ but was #{src}"
    end)

    # Check counter display
    counter = find_element(:css, ".current-photo")
    counter_text = visible_text(counter)
    assert counter_text == "1", "Counter should start at 1"

    # Verify total number of photos
    assert length(photo_slides) == 3,
      "Expected 3 photos but found #{length(photo_slides)}"

    {:ok, state}
  end

  and_ ~r/^I should see the first photo on the homepage$/, fn state ->

    # Navigate to homepage
    navigate_to("/")
    :timer.sleep(500)  # Wait for page to load

    # Find the property container
    property_div = find_element(:css, "#properties > div:first-child")

    # Find the photo container within the property
    photo_container = find_within_element(property_div, :css, ".w-48.h-48")

    # Check if there's an image
    case find_all_within_element(photo_container, :css, "img") do
      [img | _] ->
        src = attribute_value(img, "src")

        # Verify image source contains uploads path
        assert String.contains?(src, "/uploads/"),
          "Image source should contain /uploads/ but was #{src}"

        # Verify image has proper styling
        classes = attribute_value(img, "class")

        assert String.contains?(classes, "object-cover"),
          "Image should have object-cover class"
        assert String.contains?(classes, "rounded"),
          "Image should have rounded class"

        # Verify container dimensions
        container_classes = attribute_value(photo_container, "class")

        assert String.contains?(container_classes, "w-48"),
          "Container should have w-48 class"
        assert String.contains?(container_classes, "h-48"),
          "Container should have h-48 class"

      [] ->
        flunk("No image found in the photo container")
    end

    IO.puts("Homepage photo verification completed")

    {:ok, state}
  end

  and_ ~r/^I don't upload any photos$/, fn state ->
    file_input = find_element(:id, "property_photos")

    execute_script("""
      const input = document.getElementById('property_photos');
      return input.files.length;
    """)
    |> case do
      0 -> IO.puts("Confirmed: No files selected")
      count -> IO.puts("Warning: Found #{count} files when expecting none")
    end

    {:ok, state}
  end

  then_ ~r/^I should see "(?<argument_one>[^"]+)" message$/,
  fn state, %{argument_one: argument_one} ->

    assert visible_in_page?(~r/#{argument_one}/),
      "Expected to see message '#{argument_one}' but it was not found on the page"

    {:ok, state}
  end

  and_ ~r/^I upload 6 photos$/, fn state ->
    file_input = find_element(:id, "property_photos")

    execute_script("""
      (function() {
        const input = document.getElementById('property_photos');
        const files = [
          new File(['test content'], 'test_photo_1.jpg', { type: 'image/jpeg' }),
          new File(['test content'], 'test_photo_2.jpg', { type: 'image/jpeg' }),
          new File(['test content'], 'test_photo_3.jpg', { type: 'image/jpeg' }),
          new File(['test content'], 'test_photo_4.jpg', { type: 'image/jpeg' }),
          new File(['test content'], 'test_photo_5.jpg', { type: 'image/jpeg' }),
          new File(['test content'], 'test_photo_6.jpg', { type: 'image/jpeg' })
        ];
        const dataTransfer = new DataTransfer();
        files.forEach(file => dataTransfer.items.add(file));
        input.files = dataTransfer.files;
        input.dispatchEvent(new Event('change', { bubbles: true }));
      })();
    """)

    :timer.sleep(100)
    {:ok, state}
  end

  and_ ~r/^I upload 2 valid photos$/, fn state ->
    file_input = find_element(:id, "property_photos")

    execute_script("""
      (function() {
        const input = document.getElementById('property_photos');
        const files = [
          new File(['test content'], 'test_photo_1.jpg', { type: 'image/jpeg' }),
          new File(['test content'], 'test_photo_2.jpg', { type: 'image/jpeg' }),
        ];
        const dataTransfer = new DataTransfer();
        files.forEach(file => dataTransfer.items.add(file));
        input.files = dataTransfer.files;
        input.dispatchEvent(new Event('change', { bubbles: true }));
      })();
    """)

    :timer.sleep(100)
    {:ok, state}
  end

  and_ ~r/^I fill in the property advertisement form with invalid data$/, fn state ->
    fill_field({:id, "title"}, "wrong")
    fill_field({:id, "description"}, "invalid_obvs")

    # Note: These can't be set invalid from the UI
    select_dropdown("type", state[:data][:type])
    select_dropdown("property_type", state[:data][:property_type])

    fill_field({:id, "location"}, "Tartu, Estonia")
    fill_field({:id, "area"}, "-10")
    fill_field({:id, "room_count"}, "1")
    fill_field({:id, "floor"}, "1")
    fill_field({:id, "floor_count"}, "0")
    fill_field({:id, "price"}, "-100")
    {:ok, state}
  end

  then_ ~r/^I should see error messages$/, fn state ->
    assert visible_in_page? ~r/must be greater than 0/
    assert visible_in_page? ~r/Floor count must be greater than floor/
    assert visible_in_page? ~r/must be greater than 0/
    {:ok, state}
  end

  and_ ~r/^I should still see the new property advertisement form$/, fn state ->
    assert visible_in_page? ~r/Add a new property listing/
    {:ok, state}
  end

  given_ ~r/^I am not logged in$/, fn state ->
    navigate_to("/logout")
    {:ok, state}
  end

  and_ ~r/^I would like to create a new property advertisement$/, fn state ->
    navigate_to("/")
    {:ok, state}
  end

  then_ ~r/^I should not see a link to create a new property advertisement$/, fn state ->
    assert not visible_in_page? ~r/Add a new property listing/
    {:ok, state}
  end

  and_ ~r/^I should be redirected back to home page when I try to access the link$/, fn state ->
    assert current_path() == "/"
    {:ok, state}
  end

  defp setup_session(email, password) do
    navigate_to("/login")
    fill_field({:id, "email"}, email)
    fill_field({:id, "password"}, password)
    click({:id, "login_button"})
  end

  # https://stackoverflow.com/a/49861811
  defp select_dropdown(drop_down_id, option) do
    find_element(:css, "##{drop_down_id} option[value='#{option}']") |> click()
  end

  defp create_test_photo(index) do
    Path.join([
      File.cwd!(),
      "test",
      "support",
      "fixtures",
      "photo_#{index}.jpg"
    ])
  end
end
