defmodule MarkPropertyNotFavoriteContext do
  use WhiteBread.Context
  use Hound.Helpers
  alias PropTrackr.Accounts
  alias PropTrackr.Repo
  alias PropTrackr.Accounts.User
  alias PropTrackr.Properties.Property
  alias PropTrackr.Favorites.Favorite

  scenario_starting_state fn _state ->
    Ecto.Adapters.SQL.Sandbox.checkout(PropTrackr.Repo)
    Ecto.Adapters.SQL.Sandbox.mode(PropTrackr.Repo, {:shared, self()})
    Hound.start_session()
    %{}
  end

  scenario_finalize fn _status, _state ->
    Ecto.Adapters.SQL.Sandbox.checkin(PropTrackr.Repo)
    Hound.end_session()
  end

  given_ ~r/^there exists following accounts$/, fn state, %{table_data: table} ->
    table
    |> Enum.map(fn user_details -> User.changeset(%User{}, user_details) end)
    |> Enum.each(fn changeset -> Repo.insert!(changeset) end)

    random_user = List.first(table)
    owner = List.last(table)

    {
      :ok,
      state
      |> Map.put(:random_email, random_user[:email])
      |> Map.put(:random_password, random_user[:password])
      |> Map.put(:owner_email, owner[:email])
      |> Map.put(:owner_password, owner[:password])
    }
  end

  and_ ~r/^the following properties exist$/, fn state, %{ table_data: table } ->
    owner = Repo.get_by(User, email: state[:owner_email])

    advertisements =
      table
      |> Enum.map(fn details -> details |> Map.put(:reference, Ecto.UUID.generate()) end)
      |> Enum.map(fn details -> {Ecto.build_assoc(owner, :properties, details), details} end)
      |> Enum.map(fn {assoc, details} -> Property.changeset(assoc, details) end)
      |> Enum.map(fn changeset -> Repo.insert!(changeset) end)

    {
      :ok,
      state
      |> Map.put(:advertisements, advertisements)
    }
  end

  and_ ~r/^I am logged in as random user$/, fn state ->
    setup_session(state[:random_email], state[:random_password])
    {:ok, state}
  end

  and_ ~r/^the property "(?<argument_one>[^"]+)" is in my favorites$/,
  fn state, %{argument_one: argument_one} ->
    current_user = Repo.get_by(User, email: state[:random_email])

    property = Enum.find(state.advertisements, fn advert ->
      advert.title == argument_one
    end)

    favorite_changeset = %Favorite{}
    |> Favorite.changeset(%{
      user_id: current_user.id,
      property_id: property.id
    })

    Repo.insert!(favorite_changeset)

    {:ok, state}
  end

  when_ ~r/^I visit the home page$/, fn state ->
    navigate_to("/")
    {:ok, state}
  end

  when_ ~r/^I visit the details page for "(?<argument_one>[^"]+)"$/,
  fn state, %{argument_one: argument_one} ->
    property = Enum.find(state.advertisements, fn advert ->
      advert.title == argument_one
    end)

    navigate_to("/properties/#{property.reference}")
    :timer.sleep(500)

    {:ok, state}
  end

  and_ ~r/^I click the favorite button$/, fn state ->
    button = find_element(:css, ".favorite-button")
    click(button)
    :timer.sleep(500)

    reference = attribute_value(button, "data-property-reference")

    property = Enum.find(state.advertisements, fn advert ->
      advert.reference == reference
    end)

    {:ok, state |> Map.put(:last_favorited_title, property.title)}
  end


  and_ ~r/^I click the favorite button for "(?<argument_one>[^"]+)"$/,
  fn state, %{argument_one: argument_one} ->
    property = Enum.find(state.advertisements, fn advert ->
      advert.title == argument_one
    end)

    button = find_element(:css, ".favorite-button[data-property-reference='#{property.reference}']")
    click(button)
    :timer.sleep(500)

    {:ok, state |> Map.put(:last_favorited_title, argument_one)}
  end

  then_ ~r/^I should see Property removed from favorites message$/, fn state ->
    assert visible_in_page? ~r/Property removed from favorites/
    {:ok, state}
  end

  then_ ~r/^the property should be marked as not favorite$/, fn state ->
    property = case state do
      # When we've just unfavorited a property
      %{last_favorited_title: title} ->
        Enum.find(state.advertisements, fn advert ->
          advert.title == title
        end)

      # When checking a property that was never favorited
      _ ->
        List.first(state.advertisements)
    end

    button = find_element(:css, ".favorite-button[data-property-reference='#{property.reference}']")

    empty_star = "☆"
    button_text = visible_text(button)
    assert button_text == empty_star, "Expected empty star (#{empty_star}) but got #{button_text}"

    favorited_attr = attribute_value(button, "data-favorited")
    assert favorited_attr == "false", "Expected data-favorited to be 'false' but got #{favorited_attr}"

    {:ok, state}
  end



  defp setup_session(email, password) do
    navigate_to("/login")
    fill_field({:id, "email"}, email)
    fill_field({:id, "password"}, password)
    click({:id, "login_button"})
  end
end
