defmodule NotInterestedPropertiesContext do
  use WhiteBread.Context
  use Hound.Helpers
  import Ecto.Query
  alias PropTrackr.Accounts
  alias PropTrackr.Repo
  alias PropTrackr.Accounts.User
  alias PropTrackr.Properties.Property
  alias PropTrackr.NotInterested

  scenario_starting_state fn _state ->
    Ecto.Adapters.SQL.Sandbox.checkout(PropTrackr.Repo)
    Ecto.Adapters.SQL.Sandbox.mode(PropTrackr.Repo, {:shared, self()})
    Hound.start_session()
    %{}
  end

  scenario_finalize fn _status, _state ->
    Ecto.Adapters.SQL.Sandbox.checkin(PropTrackr.Repo)
    Hound.end_session()
  end

  given_ ~r/^there exists following accounts$/, fn state, %{table_data: table} ->
    table
    |> Enum.map(fn user_details -> User.changeset(%User{}, user_details) end)
    |> Enum.each(fn changeset -> Repo.insert!(changeset) end)

    random_user = List.first(table)
    owner = List.last(table)

    {
      :ok,
      state
      |> Map.put(:random_email, random_user[:email])
      |> Map.put(:random_password, random_user[:password])
      |> Map.put(:owner_email, owner[:email])
      |> Map.put(:owner_password, owner[:password])
    }
  end

  and_ ~r/^the following properties exist$/, fn state, %{ table_data: table } ->
    owner = Repo.get_by(User, email: state[:owner_email])

    advertisements =
      table
      |> Enum.map(fn details -> details |> Map.put(:reference, Ecto.UUID.generate()) end)
      |> Enum.map(fn details -> {Ecto.build_assoc(owner, :properties, details), details} end)
      |> Enum.map(fn {assoc, details} -> Property.changeset(assoc, details) end)
      |> Enum.map(fn changeset -> Repo.insert!(changeset) end)

    {
      :ok,
      state
      |> Map.put(:advertisements, advertisements)
    }
  end

  and_ ~r/^I am logged in as random user$/, fn state ->
    setup_session(state[:random_email], state[:random_password])
    {:ok, state}
  end

  and_ ~r/^I am logged in as owner$/, fn state ->
    setup_session(state[:owner_email], state[:owner_password])
    {:ok, state}
  end

  and_ ~r/^the property "(?<argument_one>[^"]+)" is marked as not interested$/,
  fn state, %{argument_one: argument_one} ->
    current_user = Repo.get_by(User, email: state[:random_email])
    property = Enum.find(state.advertisements, fn advert ->
      advert.title == argument_one
    end)

    not_interested_changeset = %NotInterested{}
    |> NotInterested.changeset(%{
      user_id: current_user.id,
      property_id: property.id
    })

    Repo.insert!(not_interested_changeset)

    {:ok, state}
  end

  when_ ~r/^I visit the home page$/, fn state ->
    navigate_to("/")
    {:ok, state}
  end

  and_ ~r/^I click interest menu for "(?<argument_one>[^"]+)"$/,
  fn state, %{argument_one: argument_one} ->
    property = Enum.find(state.advertisements, fn advert ->
      advert.title == argument_one
    end)

    # Update selector to match your HTML
    # Change this line in ManagePropertyInterestContext
    menu_button = find_element(:css, ".interest-menu-button[data-property-reference='#{property.reference}']")
    click(menu_button)
    :timer.sleep(500)

    {:ok, state}
  end

  and_ ~r/^I click "(?<option>[^"]+)" option$/, fn state, %{option: option} ->
    option_button = find_element(:css, ".interest-option[data-status='#{String.downcase(option) |> String.replace(" ", "_")}']")
    click(option_button)
    :timer.sleep(500)

    {:ok, state}
  end

  then_ ~r/^Property should move to bottom of the list$/, fn state ->
    properties = find_all_elements(:css, "#properties > div")
    last_property = List.last(properties)
    property = List.first(state.advertisements)

    title_element = find_within_element(last_property, :css, "h2")
    title_text = visible_text(title_element)

    assert title_text == property.title
    {:ok, state}
  end

  then_ ~r/^Property should move to its original position$/, fn state ->
    properties = find_all_elements(:css, "#properties > div")
    first_property = List.first(properties)
    property = List.first(state.advertisements)

    title_element = find_within_element(first_property, :css, "h2")
    title_text = visible_text(title_element)

    assert title_text == property.title
    {:ok, state}
  end

  then_ ~r/^I should see success message for marking as not interested$/, fn state ->
    assert visible_in_page?(~r/Marked as not interested/i)
    {:ok, state}
  end

  then_ ~r/^I should see success message for marking as interested$/, fn state ->
    assert visible_in_page?(~r/Marked as interested/i)
    {:ok, state}
  end

  then_ ~r/^I should not see interest menu for "(?<argument_one>[^"]+)"$/,
  fn state, %{argument_one: argument_one} ->
    property = Enum.find(state.advertisements, fn advert ->
      advert.title == argument_one
    end)

    elements = find_all_elements(:css, ".interest-menu-button[data-property-reference='#{property.reference}']")

    assert Enum.empty?(elements),
      "Expected no interest menu for property '#{argument_one}' but found #{length(elements)} menu(s)"

    {:ok, state}
  end

  # Add these to ManagePropertyInterestContext

  and_ ~r/^Property should be added to database$/, fn state ->
    property = List.first(state.advertisements)
    current_user = Repo.get_by(User, email: state[:random_email])

    not_interested = Repo.one(from n in NotInterested,
      where: n.user_id == ^current_user.id and n.property_id == ^property.id)

    assert not_interested != nil,
      "Expected property to be marked as not interested in database"

    navigate_to("/logout")
    {:ok, state}
  end

  and_ ~r/^Property should be deleted from database$/, fn state ->
    property = List.first(state.advertisements)
    current_user = Repo.get_by(User, email: state[:random_email])

    not_interested = Repo.one(from n in NotInterested,
      where: n.user_id == ^current_user.id and n.property_id == ^property.id)

    assert not_interested == nil,
      "Expected property to be removed from not interested in database"

    navigate_to("/logout")
    {:ok, state}
  end

  defp setup_session(email, password) do
    navigate_to("/login")
    fill_field({:id, "email"}, email)
    fill_field({:id, "password"}, password)
    click({:id, "login_button"})
  end
 end
