defmodule PropertyDeleteContext do
  use WhiteBread.Context
  use Hound.Helpers
  alias PropTrackr.Repo
  alias PropTrackr.Accounts.User
  alias PropTrackr.Properties.Property

  scenario_starting_state fn _state ->
    Ecto.Adapters.SQL.Sandbox.checkout(PropTrackr.Repo)
    Ecto.Adapters.SQL.Sandbox.mode(PropTrackr.Repo, {:shared, self()})
    Hound.start_session()
    %{}
  end

  scenario_finalize fn _status, _state ->
    Ecto.Adapters.SQL.Sandbox.checkin(PropTrackr.Repo)
    Hound.end_session()
  end

  given_ ~r/^there exists following accounts$/, fn state, %{table_data: table}->
    table
    |> Enum.map(fn user_details -> User.changeset(%User{}, user_details) end)
    |> Enum.each(fn changeset -> Repo.insert!(changeset) end)

    existing_user = List.first(table)
    random_user = List.last(table)

    {
      :ok,
      state
      |> Map.put(:email, existing_user[:email])
      |> Map.put(:password, existing_user[:password])
      |> Map.put(:random_email, random_user[:email])
      |> Map.put(:random_password, random_user[:password])
    }
  end

  and_ ~r/^the following properties exist$/, fn state, %{table_data: table} ->
    logged_in_user = Repo.get_by(User, email: state[:email])

    advertisements =
      table
      |> Enum.map(fn details -> details |> Map.put(:reference, Ecto.UUID.generate()) end)
      |> Enum.map(fn details -> {Ecto.build_assoc(logged_in_user, :properties, details), details} end)
      |> Enum.map(fn {assoc, details} -> Property.changeset(assoc, details) end)
      |> Enum.map(fn changeset -> Repo.insert!(changeset) end)

    {
      :ok,
      state
      |> Map.put(:advertisements, advertisements)
    }
  end

  and_ ~r/^I am logged in$/, fn state ->
    setup_session(state[:email], state[:password])
    {:ok, state}
  end

  and_ ~r/^I am logged in as a random user who does not own the advertisement$/, fn state ->
    setup_session(state[:random_email], state[:random_password])
    {:ok, state}
  end

  and_ ~r/^I am not logged in$/, fn state ->
    navigate_to("/logout")
    {:ok, state}
  end

  and_ ~r/^I navigate to my property's details page$/, fn state ->
    advertisement = List.first(state[:advertisements])
    click({:id, "edit-#{advertisement.reference}"})
    {:ok, state}
  end

  and_ ~r/^I navigate to the property's details page$/, fn state ->
    advertisement = List.first(state[:advertisements])
    click({:id, "edit-#{advertisement.reference}"})
    {:ok, state}
  end

  then_ ~r/^I should not see a delete button$/, fn state ->
    refute visible_in_page? ~r/Delete/
    {:ok, state}
  end

  when_ ~r/^I click the delete button$/, fn state ->
    click({:id, "delete-property"})
    {:ok, state}
  end

  and_ ~r/^I confirm the deletion$/, fn state ->
    accept_dialog()
    {:ok, state}
  end

  then_ ~r/^I should see a successful deletion message$/, fn state ->
    assert visible_in_page? ~r/Property deleted successfully./
    {:ok, state}
  end

  and_ ~r/^I should be redirected to my properties page$/, fn state ->
    assert current_path() == "/me/properties"
    {:ok, state}
  end

  and_ ~r/^I should not see the deleted property in the listings$/, fn state ->
    advertisement = List.first(state[:advertisements])

    refute visible_in_page?(~r/#{Regex.escape(advertisement.title)}/)
    refute visible_in_page?(~r/#{Regex.escape(advertisement.description)}/)

    assert Repo.get_by(Property, reference: advertisement.reference) == nil
    {:ok, state}
  end

  defp setup_session(email, password) do
    navigate_to("/login")
    fill_field({:id, "email"}, email)
    fill_field({:id, "password"}, password)
    click({:id, "login_button"})
  end

end
