defmodule PropTrackrWeb.PropertiesController do
  use PropTrackrWeb, :controller

  import Ecto.Query, only: [from: 2]
  alias PropTrackr.Repo
  alias PropTrackr.Properties.{Property,  Photo}
  alias PropTrackr.Favorites.Favorite
  alias PropTrackr.Uploads
  alias PropTrackr.NotInterested

  def index(conn, _params) do
    current_user = conn.assigns.current_user
    current_user_id = current_user && current_user.id

    properties = from(p in Property,
      left_join: ni in NotInterested,
      on: ni.property_id == p.id and ni.user_id == ^(current_user_id || 0),
      where: p.state == :available,
      order_by: [asc: not is_nil(ni.id), asc: p.inserted_at],
      preload: [:photos],
      select: p)
    |> Repo.all()

    not_interested = if current_user do
      Repo.all(from n in NotInterested,
        where: n.user_id == ^current_user.id,
        select: n.property_id)
      else
        []
    end

    favorites = case conn.assigns.current_user do
      nil -> []
      current_user ->
        Repo.all(
          from f in Favorite,
          where: f.user_id == ^current_user.id,
          select: f.property_id
        )
    end

    render(conn, "index.html",
    properties: properties,
    favorites: favorites,
    not_interested: not_interested,
    is_search: false,
    )
  end

  def show(conn, %{"reference" => reference}) do
    property = Repo.one(
      from p in Property,
      where: p.reference == ^reference,
      preload: [:user, :photos],
      select: p
    )

    case property do
      nil ->
        conn
        |> put_flash(:error, "Property not found.")
        |> redirect(to: ~p"/")
      property ->
        current_user = conn.assigns.current_user
        can_edit = current_user && current_user.id == property.user_id

        favorites = case current_user do
          nil -> []
          user ->
            Repo.all(
              from f in Favorite,
              where: f.user_id == ^user.id,
              select: f.property_id
            )
        end

        similar_properties = get_similar_properties(property)

        render(conn, "show.html",
          property: property,
          can_edit: can_edit,
          favorites: favorites,
          similar_properties: similar_properties,
        )
    end
  end


  def new(conn, _params) do
    changeset = Property.changeset(%Property{}, %{})
    render conn, "new.html", changeset: changeset
  end


def create(conn, %{"property" => property_params}) do
  current_user = conn.assigns.current_user

  if current_user == nil do
    conn
    |> put_flash(:error, "You are not logged in!")
    |> redirect(to: "/")
  else
    uploaded_files = Map.get(property_params, "photos", [])

    # Add reference to property params
    property_params = property_params |> Map.put("reference", Ecto.UUID.generate())

    # Clean and convert property params
    property_params_with_atoms =
      property_params
      |> Map.delete("photos")
      |> Enum.map(fn {key, value} -> {String.to_atom(key), value} end)
      |> Enum.into(%{})

    # Build property association with user
    property_assoc = Ecto.build_assoc(current_user, :properties, property_params_with_atoms)
    property_changeset = Property.changeset(property_assoc, property_params_with_atoms)

    if length(uploaded_files) < 1 or length(uploaded_files) > 5 do
      conn
      |> put_flash(:error, "Please upload between 1 and 5 photos")
      |> render("new.html", changeset: property_changeset)
    else
      Ecto.Multi.new()
      |> Ecto.Multi.insert(:property, property_changeset)
      |> Ecto.Multi.run(:photos, fn repo, %{property: property} ->

        results =
          uploaded_files
          |> Enum.with_index(1)
          |> Enum.map(fn {upload, index} ->
            case upload do
              %Plug.Upload{} = file ->
                # Save file and create photo record
                filename = Uploads.upload_file(file)

                photo_changeset = Photo.changeset(%Photo{}, %{
                  filename: filename,
                  order: index,
                  property_id: property.id
                })

                case repo.insert(photo_changeset) do
                  {:ok, photo} ->
                    IO.puts("Successfully created photo with ID: #{photo.id}")
                    {:ok, photo}
                  {:error, reason} = error ->
                    IO.puts("Failed to create photo: #{inspect(reason)}")
                    error
                end
              _ -> {:error, "Invalid file upload"}
            end
          end)

        case Enum.split_with(results, &(elem(&1, 0) == :ok)) do
          {successful, []} ->
            {:ok, Enum.map(successful, &elem(&1, 1))}
          {_, failed} ->
            {:error, "Failed to create some photos: #{inspect(failed)}"}
        end
      end)
      |> Repo.transaction()
      |> case do
        {:ok, %{property: property, photos: photos}} ->
          conn
          |> put_flash(:info, "Property created successfully!")
          |> redirect(to: ~p"/properties/#{property.reference}")

        {:error, :property, changeset, _} ->
          conn
          |> put_flash(:error, "Failed to create property")
          |> render("new.html", changeset: changeset)

        {:error, :photos, error, _} ->
          conn
          |> put_flash(:error, "Failed to upload photos: #{inspect(error)}")
          |> render("new.html", changeset: property_changeset)
      end
    end
  end
end

  def edit(conn, %{"reference" => reference}) do
    property = Repo.get_by(Property, reference: reference)

    case property do
      nil ->
        conn
        |> put_flash(:error, "Property not found.")
        |> redirect(to: ~p"/")
      property ->
        if conn.assigns.current_user && conn.assigns.current_user.id == property.user_id do
          changeset = Property.changeset(property)
          render(conn, "edit.html", property: property, changeset: changeset)
        else
          conn
          |> put_flash(:error, "You don't have permission to edit this property.")
          |> redirect(to: ~p"/properties/#{property.reference}")
        end
    end
  end

  @spec update(Plug.Conn.t(), map()) :: Plug.Conn.t()
  def update(conn, %{"reference" => reference, "property" => property_params}) do
    property = Repo.get_by(Property, reference: reference)

    case property do
      nil ->
        conn
        |> put_flash(:error, "Property not found.")
        |> redirect(to: ~p"/")

      property ->
        if conn.assigns.current_user && conn.assigns.current_user.id == property.user_id do
          case Repo.update(Property.changeset(property, property_params)) do
            {:ok, updated_property} ->
              conn
              |> put_flash(:info, "Property updated successfully.")
              |> redirect(to: ~p"/properties/#{updated_property.reference}")

            {:error, changeset} ->
              render(conn, "edit.html", property: property, changeset: changeset)
          end
        else
          conn
          |> put_flash(:error, "You don't have permission to update this property.")
          |> redirect(to: ~p"/properties/#{property.reference}")
        end
    end
   end

  def delete(conn, %{"reference" => reference}) do
    if conn.assigns.current_user == nil do
      conn
      |> put_flash(:error, "You are not logged in!")
      |> redirect(to: "/")
    else
      # property = Repo.get_by(Property, reference: reference)
      property = Repo.get_by(Property, reference: reference) |> Repo.preload(:photos)

      case property do
        nil ->
          conn
          |> put_flash(:error, "Property not found.")
          |> redirect(to: ~p"/")

        property ->
          if conn.assigns.current_user.id == property.user_id do
            Enum.each(property.photos, fn photo ->
              uploads_dir = Application.app_dir(:proptrackr, "priv/static/uploads")
              file_path = Path.join(uploads_dir, photo.filename)
              File.rm(file_path)
            end)

            case Repo.delete(property) do
              {:ok, _deleted_property} ->
                conn
                |> put_flash(:info, "Property deleted successfully.")
                |> redirect(to: ~p"/me/properties")

              {:error, _changeset} ->
                conn
                |> put_flash(:error, "Failed to delete property.")
                |> redirect(to: ~p"/properties/#{property.reference}")
            end
          else
            conn
            |> put_flash(:error, "You don't have permission to delete this property.")
            |> redirect(to: ~p"/properties/#{property.reference}")
          end
      end
    end
  end

  def update_status(conn, %{"reference" => reference, "state" => state}) do
    case conn.assigns[:current_user] do
      nil ->
        conn
        |> put_status(:unauthorized)
        |> json(%{error: "You must be logged in to perform this action"})

      current_user ->
        property = Repo.get_by(Property, reference: reference)

        case property do
          nil ->
            conn
            |> put_status(:not_found)
            |> json(%{error: "Property not found"})

          property ->
            if current_user.id == property.user_id do
              changeset = Property.state_changeset(property, %{"state" => state})

              case Repo.update(changeset) do
                {:ok, updated_property} ->
                  json(conn, %{
                    success: true,
                    message: "Status updated successfully",
                    state: updated_property.state
                  })

                {:error, _changeset} ->
                  conn
                  |> put_status(:unprocessable_entity)
                  |> json(%{error: "Failed to update status"})
              end
            else
              conn
              |> put_status(:unauthorized)
              |> json(%{error: "You don't have permission to update this property"})
            end
        end
    end
  end

  defp get_similar_properties(property) do
    Repo.all(
      from p in Property,
      where: p.state == :available
         and p.id != ^property.id,
      order_by: [
        desc: fragment("abs(? - ?)", p.price, ^property.price),
        desc: fragment("abs(? - ?)", p.room_count, ^property.room_count),
        desc: fragment("abs(? - ?)", p.area, ^property.area),
        desc: fragment("abs(? - ?)", p.floor, ^property.floor)
      ],
      limit: 5,
      select: p
    )
  end
end
