defmodule PropTrackrWeb.FavoriteControllerTest do
  use PropTrackrWeb.ConnCase
  alias PropTrackr.Accounts.User
  alias PropTrackr.Properties.Property
  alias PropTrackr.Favorites.Favorite
  alias PropTrackr.Repo
  import Ecto.Query, only: [from: 2]

  setup do
    owner = %User{
      name: "Property",
      surname: "Owner",
      birth_date: "2000-01-01",
      phone_number: "111",
      bio: "Hey",
      email: "property.owner@gmail.com",
      password: "password",
      confirm_password: "password"
    }
    owner = Repo.insert!(owner)

    random_user = %User{
      name: "Random",
      surname: "User",
      birth_date: "2000-01-01",
      phone_number: "000",
      bio: "Hi",
      email: "random.user@gmail.com",
      password: "password",
      confirm_password: "password"
    }
    random_user = Repo.insert!(random_user)

    property = %Property{
      reference: Ecto.UUID.generate(),
      title: "Test Property",
      description: "Test Description",
      type: :sell,
      property_type: :house,
      state: :available,
      location: "Test Location",
      room_count: 3,
      area: 100.0,
      floor: 2,
      floor_count: 5,
      price: 500000.0,
      user_id: owner.id
    }
    property = Repo.insert!(property)

    {:ok, %{owner: owner, random_user: random_user, property: property}}
  end

  test "authenticated user can favorite a property", %{conn: conn, random_user: user, property: property} do
    conn = conn |> setup_session(user)
    conn = post(conn, ~p"/api/properties/#{property.reference}/favorite")

    IO.inspect(conn)

    assert json_response(conn, 200)["message"] == "Property added to favorites"

    favorite = Repo.one(from f in Favorite,
      where: f.user_id == ^user.id and f.property_id == ^property.id)
    assert favorite != nil
  end

  test "authenticated user cannot favorite same property twice", %{conn: conn, random_user: user, property: property} do
    Repo.insert!(%Favorite{user_id: user.id, property_id: property.id})

    conn = conn |> setup_session(user)
    conn = post(conn, ~p"/api/properties/#{property.reference}/favorite")

    assert json_response(conn, 422)["error"] == "Already in favorites"

    #database should still only have one favorite
    favorites_count = Repo.one(from f in Favorite,
      where: f.user_id == ^user.id and f.property_id == ^property.id,
      select: count(f.id))
    assert favorites_count == 1
  end

  test "owner cannot favorite their own property", %{conn: conn, owner: owner, property: property} do
    conn = conn |> setup_session(owner)
    conn = post(conn, ~p"/api/properties/#{property.reference}/favorite")

    assert json_response(conn, 403)["error"] == "Cannot set own property as favorite"

    #no favorite should be created
    favorite = Repo.one(from f in Favorite,
      where: f.user_id == ^owner.id and f.property_id == ^property.id)
    assert favorite == nil
  end

  test "unauthenticated user cannot favorite a property", %{conn: conn, property: property} do
    initial_favorites_count = Repo.one(from f in Favorite, select: count(f.id))
    conn = post(conn, ~p"/api/properties/#{property.reference}/favorite")

    assert json_response(conn, 401)["error"] == "You're not logged in!"

    # count should not change
    final_favorites_count = Repo.one(from f in Favorite, select: count(f.id))
    assert initial_favorites_count == final_favorites_count
  end

  test "cannot favorite non-existent property", %{conn: conn, random_user: user} do
    initial_favorites_count = Repo.one(from f in Favorite, select: count(f.id))

    conn = conn |> setup_session(user)
    conn = post(conn, ~p"/api/properties/non-existent-reference/favorite")

    assert json_response(conn, 404)["error"] == "Property not found"

    #count should not change
    final_favorites_count = Repo.one(from f in Favorite, select: count(f.id))
    assert initial_favorites_count == final_favorites_count
  end

  # MARKING AS NOT FAVORITE TESTS
  test "authenticated user can unfavorite a property", %{conn: conn, random_user: user, property: property} do
    Repo.insert!(%Favorite{user_id: user.id, property_id: property.id})

    conn = conn |> setup_session(user)
    conn = delete(conn, ~p"/api/properties/#{property.reference}/favorite")

    assert json_response(conn, 200)["message"] == "Property removed from favorites"

    favorite = Repo.one(from f in Favorite,
      where: f.user_id == ^user.id and f.property_id == ^property.id)
    assert favorite == nil
  end


  test "authenticated user cannot unfavorite non-favorited property", %{conn: conn, random_user: user, property: property} do
    conn = conn |> setup_session(user)
    conn = delete(conn, ~p"/api/properties/#{property.reference}/favorite")

    assert json_response(conn, 404)["error"] == "Favorite not found"

    # Verify no changes in database
    favorites_count = Repo.one(from f in Favorite, select: count(f.id))
    assert favorites_count == 0
  end

  test "unauthenticated user cannot unfavorite a property", %{conn: conn, random_user: user, property: property} do
    Repo.insert!(%Favorite{user_id: user.id, property_id: property.id})
    initial_favorites_count = Repo.one(from f in Favorite, select: count(f.id))

    conn = delete(conn, ~p"/api/properties/#{property.reference}/favorite")

    assert json_response(conn, 401)["error"] == "You must be logged in"

    # Verify no changes in database
    final_favorites_count = Repo.one(from f in Favorite, select: count(f.id))
    assert initial_favorites_count == final_favorites_count
  end

  test "cannot unfavorite non-existent property", %{conn: conn, random_user: user} do
    conn = conn |> setup_session(user)
    conn = delete(conn, ~p"/api/properties/non-existent-reference/favorite")

    assert json_response(conn, 404)["error"] == "Property not found"
  end

  test "deleting property removes associated favorites", %{conn: conn, random_user: user, property: property} do
    Repo.insert!(%Favorite{user_id: user.id, property_id: property.id})

    Repo.delete!(property)

    favorite = Repo.one(from f in Favorite,
      where: f.user_id == ^user.id and f.property_id == ^property.id)
    assert favorite == nil
  end

  defp setup_session(conn, user) do
    conn = conn |> post("/login", email: user.email, password: user.password)
    conn = get conn, redirected_to(conn)
    conn
  end
end
