defmodule PropTrackrWeb.PropertiesControllerTest do
  use PropTrackrWeb.ConnCase
  alias PropTrackr.Accounts.User
  alias PropTrackr.Properties.{Property, Photo}
  import Ecto.Query, only: [from: 2]
  alias PropTrackr.Repo

  @valid_data %{
    title: "Apartment",
    description: "Small apartment with a beautiful view of the city center",
    type: "rent",
    property_type: "apartment",
    location: "Tartu, Estonia",
    room_count: 1,
    area: 13.0,
    floor: 2,
    floor_count: 4,
    price: 430.0,
  }

  @invalid_data %{
    title: "wrong",
    description: "invalid_obvs",
    type: "rent",
    property_type: "apartment",
    location: "Tartu, Estonia",
    room_count: 0,
    area: -1.0,
    floor: -1,
    floor_count: -1,
    price: -1.0,
  }

  @update_data %{
    title: "Updated Apartment",
    description: "Newly renovated small apartment in the heart of the city with modern amenities",
    type: "sell",
    property_type: "apartment",
    location: "Tallinn, Estonia",
    room_count: 2,
    area: 15.0,
    floor: 3,
    floor_count: 5,
    price: 500.0,
  }

  setup do
    uploads_dir = Application.app_dir(:proptrackr, "priv/static/uploads")
    File.mkdir_p!(uploads_dir)

    test_image_path = Path.join(uploads_dir, "test_image.jpg")
    File.write!(test_image_path, "fake image content")

    test_photo = %Plug.Upload{
      path: test_image_path,
      filename: "test_image.jpg",
      content_type: "image/jpeg"
    }

    user = %User{
      name: "Test",
      surname: "User",
      birth_date: "2000-01-01",
      phone_number: "000",
      bio: "Yo",
      email: "test.user@gmail.com",
      password: "testing",
      confirm_password: "testing",
    }
    user = Repo.insert!(user)

    other_user = %User{
      name: "Other",
      surname: "User",
      birth_date: "2000-01-01",
      phone_number: "111",
      bio: "Hey",
      email: "other.user@gmail.com",
      password: "testing",
      confirm_password: "testing",
    }
    other_user = Repo.insert!(other_user)

    on_exit(fn ->
      File.rm_rf!(uploads_dir)
    end)

    {:ok, %{user: user, other_user: other_user, test_photo: test_photo}}

  end

  defp assert_photo_records(property_id, expected_count) do
    query = from(p in Photo, where: p.property_id == ^property_id)
    photos = Repo.all(query)

    assert length(photos) == expected_count,
      "Expected #{expected_count} photos, but found #{length(photos)}"

    Enum.each(photos, fn photo ->
      assert photo.property_id == property_id,
        "Photo #{photo.id} has incorrect property_id"
      assert photo.filename != nil,
        "Photo #{photo.id} has no filename"
      assert photo.order != nil,
        "Photo #{photo.id} has no order"
      assert File.exists?(Path.join(Application.app_dir(:proptrackr, "priv/static/uploads"), photo.filename)),
        "Photo file #{photo.filename} does not exist in uploads directory"
    end)
  end

  test "Authenticated user should be able to create a new property advertisement with valid data", %{ conn: conn, user: user, test_photo: test_photo } do
    conn = conn |> setup_session(user)

    property_data = Map.put(@valid_data, "photos", [test_photo])
    conn = conn |> post("/properties", property: property_data)

    uuidv4_regex =  ~r/^[0-9a-f]{8}-[0-9a-f]{4}-[0-9a-f]{4}-[0-9a-f]{4}-[0-9a-f]{12}$/
    path = String.split(redirected_to(conn), "/")
    assert Enum.at(path, -1) =~ uuidv4_regex
    assert Enum.at(path, -2) == "properties"

    advertisement_reference = Enum.at(path, -1)

    conn = get conn, redirected_to(conn)
    assert html_response(conn, 200) =~ "Property created successfully!"

    advertisement = Repo.get_by!(Property, reference: advertisement_reference) |> Repo.preload(:photos)
    assert advertisement.title == @valid_data[:title]
    assert advertisement.description == @valid_data[:description]
    assert advertisement.type == String.to_atom(@valid_data[:type])
    assert advertisement.property_type == String.to_atom(@valid_data[:property_type])
    assert advertisement.location == @valid_data[:location]
    assert advertisement.room_count == @valid_data[:room_count]
    assert advertisement.area == @valid_data[:area]
    assert advertisement.floor == @valid_data[:floor]
    assert advertisement.floor_count == @valid_data[:floor_count]
    assert advertisement.price == @valid_data[:price]
    assert advertisement.user_id == user.id # check if right user is connected to advertisement


    assert_photo_records(advertisement.id, 1)

    assert length(advertisement.photos) == 1
    photo = List.first(advertisement.photos)
    assert photo.order == 1
  end

  test "Authenticated user should not be able to create an advertisement with invalid data", %{
    conn: conn,
    user: user,
    test_photo: test_photo
  } do
    conn = conn |> setup_session(user)

    invalid_data_with_photo = Map.put(@invalid_data, "photos", [test_photo])

    conn = conn |> post("/properties", property: invalid_data_with_photo)

    response = html_response(conn, 200)

    assert response =~ "must be greater than 0"
  end


  test "Authenticated user should not be able to create an advertisement with invalid floor values", %{
    conn: conn,
    user: user,
    test_photo: test_photo
  } do
    conn = conn |> setup_session(user)

    valid_base_data = %{
      title: "Valid Title Here",
      description: "This is a valid description that is definitely long enough",
      type: "rent",
      property_type: "apartment",
      location: "Tartu, Estonia",
      room_count: 2,
      area: 50.0,
      price: 500.0,
      floor: 2,
      floor_count: 1
    }

    data_with_photo = Map.put(valid_base_data, "photos", [test_photo])

    conn = conn |> post("/properties", property: data_with_photo)

    assert html_response(conn, 200) =~ "Floor count must be greater than floor"
  end


  test "Authenticated user should not be able to create an advertisement with invalid floor count data on a house listing", %{ conn: conn, user: user, test_photo: test_photo } do
    conn = conn |> setup_session(user)

    data = %{ @invalid_data | property_type: :house, floor: 2, floor_count: 1 }

    data_with_photo = Map.put(data, "photos", [test_photo])
    conn = conn |> post("/properties", property: data_with_photo)

    # Just assert that a redirect happened, don't care about data since it does not matter here
    assert redirected_to(conn) != "/properties"
  end

  test "Unauthenticated user should be redirected to the homepage", %{
    conn: conn,
    test_photo: test_photo
  } do
    property_data = Map.put(@valid_data, "photos", [test_photo])
    conn = post(conn, "/properties", property: property_data)

    assert redirected_to(conn) == "/"

    conn = get(conn, "/")
    assert html_response(conn, 200) =~ "You are not logged in!"
  end


  test "Unauthenticated user should see a message when there are no advertisements", %{
    conn: conn
  } do
    conn = get(conn, "/")
    assert html_response(conn, 200) =~ "No advertisements at the moment"
  end

  test "Unauthenticated user should see a list of all properties", %{
    conn: conn,
    user: user,
    test_photo: test_photo
  } do
    conn = conn |> setup_session(user)
    property_data = Map.put(@valid_data, "photos", [test_photo])
    conn = conn |> post("/properties", property: property_data)

    property_path = redirected_to(conn)
    conn = get(conn, property_path)
    assert html_response(conn, 200) =~ "Property created successfully!"

    property = Repo.one(Property) |> Repo.preload(:photos)
    assert_photo_records(property.id, 1)

    conn = build_conn()
    conn = get(conn, "/")

    response = html_response(conn, 200)
    assert response =~ @valid_data[:title]
    assert response =~ @valid_data[:description]
    assert response =~ @valid_data[:location]
    assert response =~ Float.to_string(@valid_data[:price])
    assert response =~ Integer.to_string(@valid_data[:room_count])
    assert response =~ Float.to_string(@valid_data[:area])


    photo = List.first(property.photos)
    assert response =~ photo.filename

    assert File.exists?(Path.join(Application.app_dir(:proptrackr, "priv/static/uploads"), photo.filename))
  end


  # #UPDATE TESTS
  test "owner can update their property with valid data", %{conn: conn, user: user} do
    property = %Property{
      reference: Ecto.UUID.generate(),
      user_id: user.id
    }
    property = property
    |> Property.changeset(@valid_data)
    |> Repo.insert!()

    conn = conn |> setup_session(user)
    conn = put(conn, ~p"/properties/#{property.reference}", property: @update_data)

    assert redirected_to(conn) == ~p"/properties/#{property.reference}"

    conn = get(conn, ~p"/properties/#{property.reference}")
    assert html_response(conn, 200) =~ "Property updated successfully"

    updated_property = Repo.get_by!(Property, reference: property.reference)
    assert updated_property.title == @update_data[:title]
    assert updated_property.description == @update_data[:description]
    assert updated_property.type == String.to_atom(@update_data[:type])
    assert updated_property.property_type == String.to_atom(@update_data[:property_type])
    assert updated_property.location == @update_data[:location]
    assert updated_property.room_count == @update_data[:room_count]
    assert updated_property.area == @update_data[:area]
    assert updated_property.floor == @update_data[:floor]
    assert updated_property.floor_count == @update_data[:floor_count]
    assert updated_property.price == @update_data[:price]
    assert updated_property.user_id == user.id
  end

  test "owner cannot update property with invalid data", %{conn: conn, user: user} do
    property = %Property{
      reference: Ecto.UUID.generate(),
      user_id: user.id
    }
    property = property
    |> Property.changeset(@valid_data)
    |> Repo.insert!()

    conn = conn |> setup_session(user)
    conn = put(conn, ~p"/properties/#{property.reference}", property: @invalid_data)

    assert html_response(conn, 200) =~ "must be greater than 0"

    unchanged_property = Repo.get_by!(Property, reference: property.reference)
    assert unchanged_property.title == @valid_data[:title]
  end

  test "non-owner cannot update property", %{conn: conn, user: user, other_user: other_user} do
    property = %Property{
      reference: Ecto.UUID.generate(),
      user_id: user.id
    }
    property = property
    |> Property.changeset(@valid_data)
    |> Repo.insert!()

    conn = conn |> setup_session(other_user)
    conn = put(conn, ~p"/properties/#{property.reference}", property: @update_data)

    assert redirected_to(conn) == ~p"/properties/#{property.reference}"
    assert get_flash(conn, :error) == "You don't have permission to update this property."

    unchanged_property = Repo.get_by!(Property, reference: property.reference)
    assert unchanged_property.title == @valid_data[:title]
  end

  test "cannot update non-existent property", %{conn: conn, user: user} do
    conn = conn |> setup_session(user)
    conn = put(conn, ~p"/properties/non-existent-reference", property: @update_data)

    assert redirected_to(conn) == "/"
    assert get_flash(conn, :error) == "Property not found."
  end

  #DELETE TESTS
  test "owner can delete their property", %{conn: conn, user: user} do
    property = %Property{
      reference: Ecto.UUID.generate(),
      user_id: user.id
    }
    property = property
    |> Property.changeset(@valid_data)
    |> Repo.insert!()

    conn = conn |> setup_session(user)
    conn = delete(conn, ~p"/properties/#{property.reference}")

    assert redirected_to(conn) == "/me/properties"
    assert get_flash(conn, :info) == "Property deleted successfully."

    assert Repo.get_by(Property, reference: property.reference) == nil
  end

  test "non-owner cannot delete property", %{conn: conn, user: user, other_user: other_user} do
    property = %Property{
      reference: Ecto.UUID.generate(),
      user_id: user.id
    }
    property = property
    |> Property.changeset(@valid_data)
    |> Repo.insert!()

    conn = conn |> setup_session(other_user)
    conn = delete(conn, ~p"/properties/#{property.reference}")

    assert redirected_to(conn) == ~p"/properties/#{property.reference}"
    assert get_flash(conn, :error) == "You don't have permission to delete this property."

    assert Repo.get_by(Property, reference: property.reference) != nil
  end

  test "cannot delete non-existent property", %{conn: conn, user: user} do
    conn = conn |> setup_session(user)
    conn = delete(conn, ~p"/properties/non-existent-reference")

    assert redirected_to(conn) == "/"
    assert get_flash(conn, :error) == "Property not found."
  end

  test "unauthenticated user cannot delete property", %{conn: conn, user: user} do
    property = %Property{
      reference: Ecto.UUID.generate(),
      user_id: user.id
    }
    property = property
    |> Property.changeset(@valid_data)
    |> Repo.insert!()

    conn = delete(conn, ~p"/properties/#{property.reference}")


    assert redirected_to(conn) == "/"
    assert get_flash(conn, :error) == "You are not logged in!"

    assert Repo.get_by(Property, reference: property.reference) != nil
  end


  # FR17 - Test if property owner details are shown
  test "Authenticated user should see the contact details of the property owner", %{conn: conn, user: user} do
    property = %Property{
      reference: Ecto.UUID.generate(),
      user_id: user.id
    }
    property = property
    |> Property.changeset(@valid_data)
    |> Repo.insert!()

    conn = conn |> setup_session(user)
    conn = get(conn, ~p"/properties/#{property.reference}")

    owner_from_db = Repo.get!(User, property.user_id)

    # Assertions to check contact details
    assert html_response(conn, 200) =~ owner_from_db.name
    assert html_response(conn, 200) =~ owner_from_db.surname
    assert html_response(conn, 200) =~ owner_from_db.phone_number
    assert html_response(conn, 200) =~ owner_from_db.email
  end

  test "Unauthenticated user should see the contact details of the property owner", %{conn: conn, user: user} do
    property = %Property{
      reference: Ecto.UUID.generate(),
      user_id: user.id
    }
    property = property
    |> Property.changeset(@valid_data)
    |> Repo.insert!()

    conn = get(conn, ~p"/properties/#{property.reference}")

    owner_from_db = Repo.get!(User, property.user_id)

    assert html_response(conn, 200) =~ owner_from_db.name
    assert html_response(conn, 200) =~ owner_from_db.surname
    assert html_response(conn, 200) =~ owner_from_db.phone_number
    assert html_response(conn, 200) =~ owner_from_db.email
  end


  # Mark as Sold / Rented / Reserved Tests
  test "owner can update property status", %{conn: conn, user: user} do
    property = %Property{
      reference: Ecto.UUID.generate(),
      user_id: user.id,
      state: :available
    }
    property = property
    |> Property.changeset(@valid_data)
    |> Repo.insert!()

    conn = conn |> setup_session(user)
    conn = post(conn, ~p"/api/properties/#{property.reference}/status", %{state: "reserved"})

    assert json_response(conn, 200) == %{
      "success" => true,
      "message" => "Status updated successfully",
      "state" => "reserved"
    }

    updated_property = Repo.get!(Property, property.id)
    assert updated_property.state == :reserved
  end

  test "non-owner cannot update property status", %{conn: conn, user: user, other_user: other_user} do
    property = %Property{
      reference: Ecto.UUID.generate(),
      user_id: user.id,
      state: :available
    }
    property = property
    |> Property.changeset(@valid_data)
    |> Repo.insert!()

    conn = conn |> setup_session(other_user)
    conn = post(conn, ~p"/api/properties/#{property.reference}/status", %{state: "reserved"})

    assert json_response(conn, 401) == %{
      "error" => "You don't have permission to update this property"
    }

    unchanged_property = Repo.get!(Property, property.id)
    assert unchanged_property.state == :available
  end

  test "unauthenticated user cannot update property status", %{conn: conn, user: user} do
    property = %Property{
      reference: Ecto.UUID.generate(),
      user_id: user.id,
      state: :available
    }
    property = property
    |> Property.changeset(@valid_data)
    |> Repo.insert!()

    conn = post(conn, ~p"/api/properties/#{property.reference}/status", %{state: "reserved"})

    assert json_response(conn, 401) == %{
      "error" => "You must be logged in to perform this action"
    }

    unchanged_property = Repo.get!(Property, property.id)
    assert unchanged_property.state == :available
  end

  test "cannot update status of non-existent property", %{conn: conn, user: user} do
    conn = conn |> setup_session(user)
    conn = post(conn, ~p"/api/properties/non-existent-reference/status", %{state: "reserved"})

    assert json_response(conn, 404) == %{
      "error" => "Property not found"
    }
  end

  test "invalid status update is rejected", %{conn: conn, user: user} do
    property = %Property{
      reference: Ecto.UUID.generate(),
      user_id: user.id,
      state: :available
    }
    property = property
    |> Property.changeset(@valid_data)
    |> Repo.insert!()

    conn = conn |> setup_session(user)
    conn = post(conn, ~p"/api/properties/#{property.reference}/status", %{state: "invalid_state"})

    assert json_response(conn, 422) == %{
      "error" => "Failed to update status"
    }

    unchanged_property = Repo.get!(Property, property.id)
    assert unchanged_property.state == :available
  end

  defp setup_session(conn, user) do
    conn = conn |> post("/login", email: user.email, password: user.password)
    conn = get conn, redirected_to(conn)
    conn
  end

end
