defmodule PropTrackrWeb.SearchControllerTest do
  use PropTrackrWeb.ConnCase
  alias PropTrackr.Accounts.User
  alias PropTrackr.Properties.Property
  alias PropTrackr.Repo

  setup do
    user = %User{
      name: "Owner",
      surname: "User",
      birth_date: "2000-01-01",
      phone_number: "000",
      bio: "Yo",
      email: "test.user@gmail.com",
      password: "testing",
      confirm_password: "testing",
    }
    user = Repo.insert!(user)

    properties = [
      %Property{
        title: "Rent property cheap",
        description: "This is a really cool property",
        type: :rent,
        property_type: :apartment,
        state: :available,
        location: "London, UK",
        room_count: 1,
        area: 90.0,
        floor: 2,
        floor_count: 8,
        price: 500.0,
      },
      %Property{
        title: "Rent property cool",
        description: "Also a really cool property",
        type: :rent,
        property_type: :apartment,
        state: :available,
        location: "London, UK",
        room_count: 3,
        area: 160.0,
        floor: 2,
        floor_count: 5,
        price: 6000.0,
      },
      %Property{
        title: "Rent property Tartu",
        description: "Also a really cool property",
        type: :rent,
        property_type: :apartment,
        state: :available,
        location: "Tartu, Estonia",
        room_count: 2,
        area: 120.0,
        floor: 1,
        floor_count: 2,
        price: 800.0,
      },
      %Property{
        title: "Buy property Tartu",
        description: "Also a really cool property",
        type: :sell,
        property_type: :apartment,
        state: :available,
        location: "Tartu, Estonia",
        room_count: 2,
        area: 120.0,
        floor: 1,
        floor_count: 2,
        price: 800000.0,
      },
      %Property{
        title: "Buy property Tallinn",
        description: "Also a really cool property",
        type: :sell,
        property_type: :apartment,
        state: :available,
        location: "Tallinn, Estonia",
        room_count: 2,
        area: 120.0,
        floor: 1,
        floor_count: 2,
        price: 800000.0,
      }
    ]
    Enum.each(properties, fn property -> Repo.insert!(property) end)

    {:ok, %{user: user, properties: properties}}
  end

  test "Searching with boundaries should show results within the boundaries", %{ conn: conn, user: user, properties: properties } do
    properties = Repo.all(Property)

    conn = post(conn, ~p"/search", %{search: %{
      type: :any,
      location: "Estonia",
      areas: ["Tartu"],
      min_price: 100.0,
      max_price: 10000000.0,
      min_rooms: 1,
      max_rooms: 100,
    }})

    location = "Tartu, Estonia"
    shown_properties = properties |> Enum.filter(fn p -> p.location == location end)
    other_properties = properties |> Enum.reject(fn p -> p.location == location end)

    result = html_response(conn, 200)
    assert result =~ "Search results"

    shown_properties |> Enum.each(fn p ->
      assert result =~ p.title
    end)

    other_properties |> Enum.each(fn p ->
      refute result =~ p.title
    end)
  end

  test "Searching with multiple areas should show results within the areas", %{ conn: conn, user: user, properties: properties } do
    properties = Repo.all(Property)

    conn = post(conn, ~p"/search", %{search: %{
      type: :any,
      location: "Estonia",
      areas: ["Tartu", "Tallinn"],
      min_price: 100.0,
      max_price: 10000000.0,
      min_rooms: 1,
      max_rooms: 100,
    }})

    locations = ["Tartu, Estonia", "Tallinn, Estonia"]
    shown_properties = properties |> Enum.filter(fn p -> Enum.member?(locations, p.location) end)
    other_properties = properties |> Enum.reject(fn p -> Enum.member?(locations, p.location) end)

    result = html_response(conn, 200)
    assert result =~ "Search results"

    shown_properties |> Enum.each(fn p ->
      assert result =~ p.title
    end)

    other_properties |> Enum.each(fn p ->
      refute result =~ p.title
    end)
  end

  test "Searching with type filter rent should show proper results", %{ conn: conn, user: user, properties: properties } do
    properties = Repo.all(Property)

    conn = post(conn, ~p"/search", %{search: %{
      type: :rent,
      location: "Estonia",
      areas: ["Tartu", "Tallinn"],
      min_price: 100.0,
      max_price: 10000000.0,
      min_rooms: 1,
      max_rooms: 100,
    }})

    locations = ["Tartu, Estonia", "Tallinn, Estonia"]
    shown_properties = properties |> Enum.filter(fn p -> p.type == :rent and Enum.member?(locations, p.location) end)
    other_properties = properties |> Enum.reject(fn p -> p.title in Enum.map(shown_properties, fn p -> p.title end) end)

    result = html_response(conn, 200)
    assert result =~ "Search results"

    shown_properties |> Enum.each(fn p ->
      assert result =~ p.title
    end)

    other_properties |> Enum.each(fn p ->
      refute result =~ p.title
    end)
  end

  test "Searching with type filter sell should show proper results", %{ conn: conn, user: user, properties: properties } do
    properties = Repo.all(Property)

    conn = post(conn, ~p"/search", %{search: %{
      type: :sell,
      location: "Estonia",
      areas: ["Tartu", "Tallinn"],
      min_price: 100.0,
      max_price: 10000000.0,
      min_rooms: 1,
      max_rooms: 100,
    }})

    locations = ["Tartu, Estonia", "Tallinn, Estonia"]
    shown_properties = properties |> Enum.filter(fn p -> p.type == :sell and Enum.member?(locations, p.location) end)
    other_properties = properties |> Enum.reject(fn p -> p.title in Enum.map(shown_properties, fn p -> p.title end) end)

    result = html_response(conn, 200)
    assert result =~ "Search results"

    shown_properties |> Enum.each(fn p ->
      assert result =~ p.title
    end)

    other_properties |> Enum.each(fn p ->
      refute result =~ p.title
    end)
  end

  test "Searching outside of the price range should show no results", %{ conn: conn, user: user, properties: properties } do
    properties = Repo.all(Property)

    conn = post(conn, ~p"/search", %{search: %{
      type: :any,
      location: "Estonia",
      areas: ["Tartu", "Tallinn"],
      min_price: 100000000.0,
      max_price: 1000000000.0,
      min_rooms: 1,
      max_rooms: 100,
    }})

    result = html_response(conn, 200)
    assert result =~ "No advertisements at the moment"
  end

  test "Searching outside of the room range should show no results", %{ conn: conn, user: user, properties: properties } do
    properties = Repo.all(Property)

    conn = post(conn, ~p"/search", %{search: %{
      type: :any,
      location: "Estonia",
      areas: ["Tartu", "Tallinn"],
      min_price: 100.0,
      max_price: 10000000.0,
      min_rooms: 100,
      max_rooms: 1000,
    }})

    result = html_response(conn, 200)
    assert result =~ "No advertisements at the moment"
  end

  test "Searching within correct price range should show proper results", %{ conn: conn, user: user, properties: properties } do
    properties = Repo.all(Property)

    min_price = 100
    max_price = 800

    conn = post(conn, ~p"/search", %{search: %{
      type: :any,
      location: "Estonia",
      areas: ["Tartu", "Tallinn"],
      min_price: min_price,
      max_price: max_price,
      min_rooms: 1,
      max_rooms: 100,
    }})

    locations = ["Tartu, Estonia", "Tallinn, Estonia"]
    shown_properties = properties |> Enum.filter(fn p -> p.price >= min_price and p.price <= max_price and Enum.member?(locations, p.location) end)
    other_properties = properties |> Enum.reject(fn p -> p.title in Enum.map(shown_properties, fn p -> p.title end) end)

    result = html_response(conn, 200)
    assert result =~ "Search results"

    shown_properties |> Enum.each(fn p ->
      assert result =~ p.title
    end)

    other_properties |> Enum.each(fn p ->
      refute result =~ p.title
    end)
  end

  test "Searching within correct room count range should show proper results", %{ conn: conn, user: user, properties: properties } do
    properties = Repo.all(Property)

    min_rooms = 1
    max_rooms = 2

    conn = post(conn, ~p"/search", %{search: %{
      type: :any,
      location: "Estonia",
      areas: ["Tartu", "Tallinn"],
      min_price: 100.0,
      max_price: 10000000.0,
      min_rooms: min_rooms,
      max_rooms: max_rooms,
    }})

    locations = ["Tartu, Estonia", "Tallinn, Estonia"]
    shown_properties = properties |> Enum.filter(fn p -> p.room_count >= min_rooms and p.room_count <= max_rooms and Enum.member?(locations, p.location) end)
    other_properties = properties |> Enum.reject(fn p -> p.title in Enum.map(shown_properties, fn p -> p.title end) end)

    result = html_response(conn, 200)
    assert result =~ "Search results"

    shown_properties |> Enum.each(fn p ->
      assert result =~ p.title
    end)

    other_properties |> Enum.each(fn p ->
      refute result =~ p.title
    end)
  end

  test "Searching without any areas should show all results", %{ conn: conn, user: user, properties: properties } do
    properties = Repo.all(Property)

    conn = post(conn, ~p"/search", %{search: %{
      type: :any,
      location: "Estonia",
      areas: [],
      min_price: 100.0,
      max_price: 10000000.0,
      min_rooms: 1,
      max_rooms: 100,
    }})

    result = html_response(conn, 200)
    assert result =~ "Search results"

    properties |> Enum.each(fn p ->
      assert result =~ p.title
    end)
  end

  test "Searching with non-existent with type but existing location should show no results", %{ conn: conn, user: user, properties: properties } do
    properties = Repo.all(Property)

    conn = post(conn, ~p"/search", %{search: %{
      type: :sell,
      location: "UK",
      areas: ["London"],
      min_price: 100.0,
      max_price: 10000000.0,
      min_rooms: 1,
      max_rooms: 100,
    }})

    result = html_response(conn, 200)
    assert result =~ "No advertisements at the moment"
  end

end
